/******************************************************************************
 * (c) Copyright 2002,2005, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package com.ten60.netkernel.cache.se.representation2;

import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestScopeLevel;

/** Key used for lookup into the cache
 * @author Tony Butterfield
 */
public class GetKey
{
	private final String mURIString;
	private final Class mRepresentationClass;
	private final IRequestScopeLevel mScope;
	private final int mVerb;
	private final int mHash;
	
	public GetKey(IRequest aRequest, int aDepth)
	{	mURIString = aRequest.getIdentifier().toString();
		mVerb=aRequest.getVerb();
		mRepresentationClass = aRequest.getRepresentationClass();
		mScope = aRequest.getRequestScope();
		
		mHash=ConcurrentCache.primaryTableHash(mURIString, mScope, aDepth);
	}
	
	public int hashCode()
	{	return mHash;
	}
	
	// on a get this is the request and other is the cached item
	public boolean equals(Object aOther)
	{	boolean result=false;
		Key other = (Key) aOther;
		if (mURIString.equals(other.getURI().toString()) && mVerb==other.getItem().getVerb() &&
				other.getItem().isSuitableRepresentationFor(mRepresentationClass))
		{	result = true;
			IRequestScopeLevel rctx1 = mScope;
			IRequestScopeLevel rctx2 = other.getScope();
			int d=other.getDepth();
			for (int i = 0; i <= d; i++)
			{	if (rctx1==null || !rctx1.getSpace().equals(rctx2.getSpace()))
				{	result = false;
					break;
				}
				rctx1 = rctx1.getParent();
				rctx2 = rctx2.getParent();
			}
			if (result && other.hasNullTailScope())
			{	// the context of this request must end too if it is too match
				if (rctx1!=null)
				{	result=false;
				}
			}
		}
		return result;
	}
	
	public String toString()
	{	StringBuffer sb=new StringBuffer(128);
		sb.append(mURIString);
		sb.append("        ");
		IRequestScopeLevel rc = mScope;
		while(rc!=null)
		{	sb.append(rc.getSpace());
			sb.append("...");
			rc=rc.getParent();
		}
		sb.append("        -> ");
		sb.append(" ("+mRepresentationClass+")");
		return sb.toString();
	}	
}