package com.ten60.netkernel.cache.se.representation2;

import java.util.concurrent.atomic.AtomicInteger;

import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;

public class NKey
{

	private final IIdentifier mIdentifier;
	private final int mVerb;
	private final ISpace mHome;
	private final int mDepth;
	private final AtomicInteger mCount;
	private final int mHash;
	
	public NKey(IIdentifier aIdentifier, int aVerb, ISpace aHome, int aDepth)
	{
		mIdentifier=aIdentifier;
		mVerb=aVerb;
		mHome=aHome;
		
		mDepth=aDepth;
		mCount=new AtomicInteger();
		int hash=mIdentifier.toString().hashCode();
		hash^=mVerb;
		hash^=mHome.hashCode();
		mHash=hash;
	}
	
	public NKey(IIdentifier aIdentifier, int aVerb, ISpace aHome)
	{
		mIdentifier=aIdentifier;
		mVerb=aVerb;
		mHome=aHome;
		mDepth=0;
		mCount=null;
		
		int hash=mIdentifier.toString().hashCode();
		hash^=mVerb;
		hash^=mHome.hashCode();
		mHash=hash;
	}
	
	
	public int hashCode()
	{	return mHash;
	}
	
	public boolean equals(Object aOther)
	{	boolean match=aOther instanceof NKey;
		if (match)
		{	NKey other = (NKey)aOther;
			match=mIdentifier.toString().equals(other.mIdentifier.toString()) && (mVerb==other.mVerb) && mHome.equals(other.mHome);
		}
		return match;
	}
	
	public void reference()
	{	mCount.incrementAndGet();
	}
	
	public boolean dereferenceAndIsDead()
	{	return mCount.decrementAndGet()==0;
	}
	
	public int getDepth()
	{	return mDepth;
	}
	
	public IIdentifier getIdentifier()
	{	return mIdentifier;
	}
	
	public int getVerb()
	{	return mVerb;
	}
	
	public ISpace getHome()
	{	return mHome;
	}
}
